

%paths to directory containing .wav files and path for saving the parameter
%table
parentDirectory = "C:/YourFolder";
Savepath = "C:/YourParameterTable.xlsx";
extension = ".wav";

%overwrite existing table at Savepath
overwriteTable=true;

%create empty table
tableColumns = {'FileName', 'Dur', 'PF', 'Q50', 'SF_{Q50}', 'Q50_2', ...
    'Q50_{10}', 'Q50_{min}', 'Q50_n', 'Q25', 'SF', 'Flux', 'RMSI', 'Q50W'};
ParameterTable = cell2table(cell(0,length(tableColumns)), 'VariableNames', tableColumns);
newrow = cell(1,length(tableColumns));

        
%get all .wav files
Files=dir(parentDirectory);
for k=length(Files):-1:1
    if (isempty(strfind(Files(k).name,extension))) 
        Files(k)=[];
    end  
end
        
for k= 1:length(Files)

   %read new file 
   FileName=Files(k).name;

   %get file name for first column of saving table
   [ ~, newrow{1}] = fileparts(FileName); 

   %read, normalize and (if necessary) transpose audio
   [y,Fs] = audioread(Files(k).folder + "\" + FileName, 'native');

   %in case of a multi channel file, select the first channel
   if size(y,1) > 1 && size(y,2) > 1  
       if size(y,1) > size(y,2)
           y=y(:,1);
       else
           y=y(1,:);
       end
   end


   if size(y,1) > 1 %transpose if wrong vecor format
       y=y';
   end

   %normalize
   y=double(y);
   y=y./max(abs(y));

   %calculate parameters
   [ParaValsT, ~] = TalletPigParameters(y, Fs);
   [ParaValsG, ~] = GarciaPigParameters(y, Fs);
   [ParaValsL, ~] = LinhartPigParameters(y, Fs);

   %add parameters to correct row positions
   newrow{2} = ParaValsT(8);  %duration
   %Tallet
   newrow{3} = ParaValsT(1);  %peak frequency
   newrow{4} = ParaValsT(2);  %Q50
   newrow{5} = ParaValsT(3);  %Spectral Flatness
   newrow{6} = ParaValsT(4);  %Q50 start
   newrow{7} = ParaValsT(5);  %Q50 end
   newrow{8} = ParaValsT(6); %Q50 min
   newrow{9} = ParaValsT(7); %Q50 max window number
   %Garcias
   newrow{10} = ParaValsG(1); %Q25
   newrow{11} = ParaValsG(2); %Spectral Flatness
   newrow{12} = ParaValsG(3); %Spectral Flux
   %Linhart
   newrow{13} = ParaValsL(1); %RMSI (only relative value)
   newrow{14} = ParaValsL(2); %Q50

   %add row to table
   ParameterTable = [ParameterTable;newrow];

end

if overwriteTable
        writetable(ParameterTable,Savepath);
end

function [ParaVals, paraNames] = LinhartPigParameters(y, Fs)
% CALCULATES PARAMETERS BASED ON A PAPER OF LINHART ET AL.:
% Expression of Emotional Arousal in Two Different Piglet Call Types
% (2015)
% https://doi.org/10.1371/journal.pone.0135414
%
% y = audio signal
% Fs = audio sampling rate

paraNames=["","",""];
ParaVals = zeros(1,3);

%window length and window overlap
wl = 1024;
ovl = 0.5; %derived from http://www.avisoft.com/SASLabPro.pdf

[~, f, ~, specs] = windowedFFT(y,Fs,wl,ovl, 'hamming');

%average energy spectrum
av_spec=mean(specs,1);


%RMSI of sound intensity
RMSI = sqrt(mean(y.^2));

paraNames(1)="root mean square of sound intensity";
ParaVals(1)=RMSI;


%central frequency of average (enery) spectrum
spectotal=sum(av_spec);

for k=1:wl
    
   if sum(av_spec(1:k))>0.5*spectotal
      Q50=f(max(k-1,1));
      break;
   end
    
end

paraNames(2)="Q50";
ParaVals(2)=Q50;


%duration (same as Tallet)
paraNames(3)="duration";
ParaVals(3)=length(y)*(1/Fs);


end


function [ParaVals, paraNames] = TalletPigParameters(y, Fs)
% CALCULATES PARAMETERS BASED ON A PAPER OF TALLET ET AL.:
% Encoding of Situations in the Vocal Repertoire of Piglets
% (Sus scrofa): A Comparison of Discrete and Graded Classifications
% (2013)
% https://doi.org/10.1371/journal.pone.0071841
%
% y = audio signal
% Fs = audio sampling rate

paraNames=["","","","","","","",""];
ParaVals = zeros(1,8);

%initializations
L=length(y);
wl=1024;
hwl=floor(wl/2)+1; %half of window length
ovl=0.875;

%calculate eleven partial windows
n_win = 11;  
start_pos=floor(1:(L-wl+1)/(n_win-1):(L-wl+1));
if length(start_pos)<n_win
    start_pos=[start_pos,L-wl+1];
end
y_windows=zeros(n_win-2,wl);   %exclude first and last window
specs9=zeros(n_win-2,hwl);  
for k=2:n_win-1
    y_windows(k-1,:)=y(start_pos(k):(start_pos(k)+wl-1));
    %will only calculate one window for the entire input signal (y section)
    [~, f9, ~, specs9(k-1,:)] = windowedFFT(y_windows(k-1,:),Fs,wl,0,'hamming'); 
end

%calculate energy spectrum of entire signal as one large window
[~, f, ~, spec] = windowedFFT(y,Fs,L,ovl,'hamming'); 

hwl9=size(specs9,2); %half window length
hwl=size(spec,2);

%peak frequency

%position of maximum energy
[~,pos1] = max(spec);

%frequency at this position
paraNames(1)="peak frequency";
ParaVals(1)=f(pos1);


%q50 and spectral flatness for all windows
q50total=0;
SF=0;


half = sum(spec(:))/2;
for k=1:hwl
    if(sum(spec(1:k))>half)
        q50total=f(max(k-1,1)); %last position before half of area was exceeded
        break;
    end
end

%calculate SF
SF=log((geomean(spec))/(mean(spec)));



paraNames(2)="q50";
ParaVals(2)=q50total;
paraNames(3)="Spectral Flatness";
ParaVals(3)=SF./q50total;  %scale SF to q50 

%calculate q50 for 9 partial windows
q50win9=zeros(1,size(specs9,1));

for g=1:size(specs9,1)
    half = sum(specs9(g,:))/2;
    for k=1:hwl9
        if(sum(specs9(g,1:k))>half)
            q50win9(g)=f9(max(k-1,1)); %last position before half of area was exceeded
            break;
        end
    end
end


paraNames(4)="q50start";
ParaVals(4)=q50win9(1);
paraNames(5)="q50end";
ParaVals(5)=q50win9(end);
paraNames(6)="q50min";
ParaVals(6)=min(q50win9);

[~,pos] = max(q50win9);

paraNames(7)="q50maxpos";
ParaVals(7)=pos;

%duration
paraNames(8)="duration";
ParaVals(8)=length(y)*(1/Fs);

end


function [ParaVals, paraNames] = GarciaPigParameters(y, Fs)
% CALCULATES PARAMETERS BASED ON A PAPER OF GARCIA ET AL.:
% Structural Classification of Wild Boar (Sus scrofa) Vocalizations
% (2015)
% https://doi.org/10.1111/eth.12472
%
% y = audio signal
% Fs = audio sampling rate

paraNames=["","","",""];
ParaVals = zeros(1,4);

%window length and window overlap
wl=1024;
ovl=0.5;

[~, f, ffts, specs] = windowedFFT(y,Fs,wl,ovl, 'hamming');

W_num = size(ffts,1);  %number of windows
wlh = size(ffts,2);    %half window length



%Quartile 25 

Q25s = zeros(1,W_num);
spectotals = sum(specs,2);

for k=1:W_num
    for g=1:wlh
       if sum(specs(k,1:g))>0.25*spectotals(k)
          Q25s(k)=f(max(g-1,1));
          break;
       end
    end
end

%average Q25 quartiles fo all windows
Q25= mean(Q25s);  

paraNames(1)="Q25";
ParaVals(1)=Q25;


%Spectral flatness and spectral flux

FLATs=zeros(1,W_num);
SFs=zeros(1,W_num-1);
for k=1:W_num
   if k<W_num
      SFs(k)= sum((ffts(k+1,:)./sum(ffts(k+1,:))-ffts(k,:)./sum(ffts(k,:))).^2);
   end
   FLATs(k)=log((geomean(ffts(k,:)))/(mean(ffts(k,:)))); 
end

FLAT = mean(FLATs);
SF = mean(SFs);

paraNames(2)="Spectral Flatness";
ParaVals(2)=FLAT;
paraNames(3)="Spectral Flux";
ParaVals(3)=SF;

%duration (same as Tallet)

paraNames(4)="duration";
ParaVals(4)=length(y)*(1/Fs);

end



function [fftAverage, f, ffts, spec] = windowedFFT(y,Fs,wl,ovl, winType)
%CALCULATES DIFFERENT KINDS OF FFTS FOR A SIGNAL
%Input
%   y = signal
%   wl = window length in samples
%   ovl = overlap as fraction (between 0 and 1)
%   Fs = sampling frequency
%Output
%   fftAverage = average of all amplitude spectra
%   f = frequency vector of windows
%   ffts = amplitude spectra for all windows
%   spec = energy spectra for all windows


%correct vector format if needed
if size(y,1) > 1 && size(y,2) > 1    
    ME = MException('MyComponent:wrongType', ...
        'Variable y cannot be in matrix format');
    throw(ME);
elseif size(y,1) > 1 %transpose if wrong vecor format
    y=y';
end


hwl=floor(wl/2)+1; %half window length 
shift=floor(wl*(1-ovl));  %shift in data points for each window
N_win = floor((length(y)- wl)/(shift)) +1; %number of windows

fftAverage=zeros(1, hwl);
ffts = zeros(N_win, hwl);
spec = zeros(N_win, hwl);
for h=1:N_win
    %windowing, no window (rectangle window) is default
    if strcmp(winType,'hanning')
        win=double(y((h-1)*(shift)+1:((h-1)*(shift)+1)+wl-1)).*hanning(wl)'; 
    elseif strcmp(winType,'hamming')
        win=double(y((h-1)*(shift)+1:((h-1)*(shift)+1)+wl-1)).*hamming(wl)';
    else
        win=double(y((h-1)*(shift)+1:((h-1)*(shift)+1)+wl-1));
    end
    winfft0=abs(fft(win));
    ffts(h,:)=winfft0(1:hwl)./wl; 
    ffts(h,2:end)=ffts(h,2:end).*2; 
    %energy is amplitude spectrum squared
    spec(h,:)=(winfft0(1:hwl).^2)./wl;
    spec(h,2:end)=spec(h,2:end).*2; 
    fftAverage=fftAverage+(winfft0(1:hwl)); %add to average fft
end
fftAverage=(fftAverage./(N_win)); %divide by number of windows to get average

%calculate frequency vector
vec=0:1:wl-1;
f = vec.*(Fs/wl);
f = f(1:hwl)';



end